"""
    Declare DateBound, a date comparison utility class.
"""

from DateTime import DateTime

class DateBound:
    """
    """
    HIGHEST = []    # marker value for instances which are *always* higher
    LOWEST  = []    # marker value for instances which are *always* lower

    def __init__( self, value=None, **kw ):
        """
            Initialize self._value to one of:

             * HIGHEST (if value passed is None, and kw['default_high'] != 0)

             * LOWEST (if value passed is None, and kw['default_high'] == 0)

             * An integer representing seconds since epoch (try fairly
               hard to convert, using DateTime to parse strings).
        """
        default_high = kw.get( 'default_high', 1 )

        if type( value ) == type( '' ):
            try: value = DateTime( value )  # Use DT's parsing
            except: value = None
        
        if isinstance( value, DateTime ):
            value = value.timeTime()
        
        if isinstance( value, DateBound ):
            self._value = value._value

        elif value is None:
            if default_high:
                self._value = self.HIGHEST
            else:
                self._value = self.LOWEST
        else:
            self._value = int( value )

    def __cmp__( self, rhs ):
        """
            Implement special comparison semantics of:
            
             * HIGHEST (always compares higher than anything except itself)

             * LOWEST (always compares lower than anything except itself)

            Deal with possibility that rhs is a DateBound, too.
        """
        rhs = DateBound( rhs )

        if self._value is rhs._value:
            return 0

        if self._value is self.HIGHEST:
            return 1

        if self._value is self.LOWEST:
            return -1

        if rhs._value is self.HIGHEST:
            return -1

        if rhs._value is self.LOWEST:
            return 1

        if self._value == rhs._value:
            return 0
        
        return self._value > rhs._value and 1 or -1

    def __rcmp__( self, lhs ):
        """
            Invert the comparison, coz we are on the RHS.
        """
        return -( self.__cmp__( lhs ) )

class LowerDateBound( DateBound ):
    """
        Convenience wrapper for bounds which default low.
    """
    def __init__( self, value=None ):
        DateBound.__init__( self, value, default_high=0 )

class UpperDateBound( DateBound ):
    """
        Convenience wrapper for bounds which default high.
    """
    def __init__( self, value=None ):
        DateBound.__init__( self, value, default_high=1 )
