##############################################################################
# 
# Zope Public License (ZPL) Version 1.0
# -------------------------------------
# 
# Copyright (c) Digital Creations.  All rights reserved.
# 
# This license has been certified as Open Source(tm).
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
# 
# 1. Redistributions in source code must retain the above copyright
#    notice, this list of conditions, and the following disclaimer.
# 
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions, and the following disclaimer in
#    the documentation and/or other materials provided with the
#    distribution.
# 
# 3. Digital Creations requests that attribution be given to Zope
#    in any manner possible. Zope includes a "Powered by Zope"
#    button that is installed by default. While it is not a license
#    violation to remove this button, it is requested that the
#    attribution remain. A significant investment has been put
#    into Zope, and this effort will continue if the Zope community
#    continues to grow. This is one way to assure that growth.
# 
# 4. All advertising materials and documentation mentioning
#    features derived from or use of this software must display
#    the following acknowledgement:
# 
#      "This product includes software developed by Digital Creations
#      for use in the Z Object Publishing Environment
#      (http://www.zope.org/)."
# 
#    In the event that the product being advertised includes an
#    intact Zope distribution (with copyright and license included)
#    then this clause is waived.
# 
# 5. Names associated with Zope or Digital Creations must not be used to
#    endorse or promote products derived from this software without
#    prior written permission from Digital Creations.
# 
# 6. Modified redistributions of any form whatsoever must retain
#    the following acknowledgment:
# 
#      "This product includes software developed by Digital Creations
#      for use in the Z Object Publishing Environment
#      (http://www.zope.org/)."
# 
#    Intact (re-)distributions of any official Zope release do not
#    require an external acknowledgement.
# 
# 7. Modifications are encouraged but must be packaged separately as
#    patches to official Zope releases.  Distributions that do not
#    clearly separate the patches from the original work must be clearly
#    labeled as unofficial distributions.  Modifications which do not
#    carry the name Zope may be packaged in any form, as long as they
#    conform to all of the clauses above.
# 
# 
# Disclaimer
# 
#   THIS SOFTWARE IS PROVIDED BY DIGITAL CREATIONS ``AS IS'' AND ANY
#   EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
#   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
#   PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL DIGITAL CREATIONS OR ITS
#   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
#   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
#   USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
#   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
#   OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
#   OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
#   SUCH DAMAGE.
# 
# 
# This software consists of contributions made by Digital Creations and
# many individuals on behalf of Digital Creations.  Specific
# attributions are listed in the accompanying credits file.
# 
##############################################################################

"""Basic membership tool.
$Id: MembershipTool.py,v 1.4 2000/09/15 16:05:37 shane Exp $
"""
__version__='$Revision: 1.4 $'[11:-2]


from utils import UniqueObject, _getAuthenticatedUser, _checkPermission
from OFS.SimpleItem import SimpleItem
from Globals import default__class_init__
from AccessControl.User import nobody


class MembershipTool (SimpleItem, UniqueObject):
    # This tool accesses member data through an acl_users object.
    # It can be replaced with something that accesses member data in
    # a different way.
    id = 'portal_membership'
    meta_type = 'PTK Basic Membership Tool'

    __ac_permissions__=(
        ('Manage portal', ('getMemberById', 'listMemberIds', 'listMembers',)),
        )

    getAuthenticatedMember__roles__ = None  # Allow all.
    def getAuthenticatedMember(self):
        '''
        Returns the currently authenticated member object
        or the Anonymous User.  Never returns None.
        '''
        u = _getAuthenticatedUser(self)
        if u is None:
            u = nobody.__of__(self.acl_users)
        if hasattr(u, 'aq_base'):
            base = u.aq_base
        else:
            base = u
            u = base.__of__(self.acl_users)

        if not hasattr(base, 'getMemberId'):
            # Authenticated outside the portal.
            if _checkPermission('Manage portal', self):
                # This user is allowed to manage the portal,
                # so we can assume the user can take on a special
                # identity within the portal.
                name = u.getUserName()
                try:
                    m = self.getMemberById(name)
                    if m is not None:
                        return m
                except:
                    # No corresponding identity exists.
                    pass
            # Try to wrap the user with preferences.
            u = self.wrapUserWithPreferences(u) or u
        return u

    wrapUserWithPreferences__roles__ = ()  # No permission.
    def wrapUserWithPreferences(self, u):
        '''
        Provides an opportunity for a portal_preferences tool
        to retrieve and store user preferences independently of
        the user object.
        '''
        parent = self.aq_inner.aq_parent
        base = parent.aq_base
        if hasattr(base, 'portal_preferences'):
            # Get portal_preferences to do the wrapping.
            prefs = parent.portal_preferences
            try:
                return prefs.wrapUser(u)
            except:
                pass
        # Failed.
        return None        

    isAnonymousUser__roles__ = None  # Allow all.
    def isAnonymousUser(self):
        '''
        Returns 1 if the user is not logged in.
        '''
        u = _getAuthenticatedUser(self)
        if u is None or u.getUserName() == 'Anonymous User':
            return 1
        return 0

    checkPermission__roles__ = None  # Allow all.
    def checkPermission(self, permissionName, object, subobjectName=None):
        '''
        Checks whether the current user has the given permission on
        the given object or subobject.
        '''
        if subobjectName is not None:
            object = getattr(object, subobjectName)
        return _checkPermission(permissionName, object)

    credentialsChanged__roles__ = None  # Allow all.
    def credentialsChanged(self, password):
        '''
        Notifies the authentication mechanism that this user has changed
        passwords.  This can be used to update the authentication cookie.
        Note that this call should *not* cause any change at all to user
        databases.
        '''
        if not self.isAnonymousUser():
            acl_users = self.acl_users
            if hasattr(acl_users.aq_base, 'credentialsChanged'):
                # Take advantage of an interface provided by LoginManager.
                user = _getAuthenticatedUser(self)
                id = user.getUserName()
                acl_users.credentialsChanged(user, id, password)

    def getMemberById(self, id):
        '''
        Returns the given member.
        '''
        u = self.acl_users.getUser(id)
        if u is not None:
            if hasattr(u, 'aq_base'):
                base = u
                u = u.__of__(self.acl_users)
            else:
                base = u.aq_base
            if not hasattr(base, 'getMemberId'):
                # Try to wrap the user with preferences.
                u = self.wrapUserWithPreferences(u) or u
        return u

    def __getPUS(self):
        # Gets something we can call getUsers() and getUserNames() on.
        acl_users = self.acl_users
        if hasattr(acl_users, 'getUsers'):
            return acl_users
        else:
            # This hack works around the absence of getUsers() in LoginManager.
            # Get the PersistentUserSource object that stores our users
            for us in acl_users.UserSourcesGroup.objectValues():
                if us.meta_type == 'Persistent User Source':
                    return us.__of__(acl_users)

    def listMemberIds(self):
        '''Lists the ids of all members.  This may eventually be
        replaced with a set of methods for querying pieces of the
        list rather than the entire list at once.
        '''
        return self.__getPUS().getUserNames()
    
    def listMembers(self):
        '''Gets the list of all members.
        '''
        return self.__getPUS().getUsers()

    addMember__roles__ = ()  # No permission.
    def addMember(self, id, password, roles, domains):
        '''Adds a new member to the user folder.  Security checks will have
        already been performed.  Called by portal_registration.
        '''
        acl_users = self.acl_users
        if hasattr(acl_users, '_addUser'):
            acl_users._addUser(id, password, password, roles, domains)
        else:
            # The acl_users folder is a LoginManager.  Search for a UserSource
            # with the needed support.
            for source in acl_users.UserSourcesGroup.objectValues():
                if hasattr(source, 'addUser'):
                    source.__of__(self).addUser(id, password, roles, domains)
                    return
            raise "Can't add Member", "No supported UserSources"

    listActions__roles__ = ()  # No permission.
    def listActions(self, info):
        return None

    getHomeFolder__roles__ = None # Anonymous permission
    def getHomeFolder(self, id=None, verifyPermission=0):
        """Returns a member's home folder object or None.
        Set verifyPermission to 1 to return None when the user
        doesn't have the View permission on the folder.
        """
        return None
        
    getHomeUrl__roles__ = None # Anonymous permission
    def getHomeUrl(self, id=None, verifyPermission=0):
        """Returns the URL to a member's home folder or None.
        Set verifyPermission to 1 to return None when the user
        doesn't have the View permission on the folder.
        """
        return None


default__class_init__(MembershipTool)
