##############################################################################
# 
# Zope Public License (ZPL) Version 1.0
# -------------------------------------
# 
# Copyright (c) Digital Creations.  All rights reserved.
# 
# This license has been certified as Open Source(tm).
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
# 
# 1. Redistributions in source code must retain the above copyright
#    notice, this list of conditions, and the following disclaimer.
# 
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions, and the following disclaimer in
#    the documentation and/or other materials provided with the
#    distribution.
# 
# 3. Digital Creations requests that attribution be given to Zope
#    in any manner possible. Zope includes a "Powered by Zope"
#    button that is installed by default. While it is not a license
#    violation to remove this button, it is requested that the
#    attribution remain. A significant investment has been put
#    into Zope, and this effort will continue if the Zope community
#    continues to grow. This is one way to assure that growth.
# 
# 4. All advertising materials and documentation mentioning
#    features derived from or use of this software must display
#    the following acknowledgement:
# 
#      "This product includes software developed by Digital Creations
#      for use in the Z Object Publishing Environment
#      (http://www.zope.org/)."
# 
#    In the event that the product being advertised includes an
#    intact Zope distribution (with copyright and license included)
#    then this clause is waived.
# 
# 5. Names associated with Zope or Digital Creations must not be used to
#    endorse or promote products derived from this software without
#    prior written permission from Digital Creations.
# 
# 6. Modified redistributions of any form whatsoever must retain
#    the following acknowledgment:
# 
#      "This product includes software developed by Digital Creations
#      for use in the Z Object Publishing Environment
#      (http://www.zope.org/)."
# 
#    Intact (re-)distributions of any official Zope release do not
#    require an external acknowledgement.
# 
# 7. Modifications are encouraged but must be packaged separately as
#    patches to official Zope releases.  Distributions that do not
#    clearly separate the patches from the original work must be clearly
#    labeled as unofficial distributions.  Modifications which do not
#    carry the name Zope may be packaged in any form, as long as they
#    conform to all of the clauses above.
# 
# 
# Disclaimer
# 
#   THIS SOFTWARE IS PROVIDED BY DIGITAL CREATIONS ``AS IS'' AND ANY
#   EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
#   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
#   PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL DIGITAL CREATIONS OR ITS
#   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
#   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
#   USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
#   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
#   OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
#   OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
#   SUCH DAMAGE.
# 
# 
# This software consists of contributions made by Digital Creations and
# many individuals on behalf of Digital Creations.  Specific
# attributions are listed in the accompanying credits file.
# 
##############################################################################

from ExtensionClass import Base
import Globals
try: from OFS.ObjectManager import UNIQUE
except: UNIQUE = 2


class ImmutableId (Base):
    def _setId(self, id):
        if id != self.id:
            raise Globals.MessageDialog(
                title='Invalid Id',
                message='Cannot change the id of this object',
                action ='./manage_main',)


class UniqueObject (ImmutableId):
    __replaceable__ = UNIQUE


try:
    # Zope 2.2.x access control.
    from AccessControl import getSecurityManager
    def _getAuthenticatedUser(self):
        return getSecurityManager().getUser()
    def _checkPermission(permission, obj):
        return getSecurityManager().checkPermission(permission, obj)
except:
    # Zope 2.1.x access control.
    from AccessControl import User
    def _getAuthenticatedUser(self):
        u = self.REQUEST.get('AUTHENTICATED_USER', None)
        if u is None:
            u = User.nobody
        return u
    def _checkPermission(permission, obj):
        return getAuthenticatedUser(obj).has_permission(permission, obj)


# If Zope ever provides a call to getRolesInContext() through
# the SecurityManager API, the method below needs to be updated.
def limitGrantedRoles(roles, context, special_roles=()):
    # Only allow a user to grant roles already possessed by that user,
    # with the exception that all special_roles can also be granted.
    user = _getAuthenticatedUser(context)
    if user is None:
        user_roles = ()
    else:
        user_roles = user.getRolesInContext(context)
    if 'Manager' in user_roles:
        # Assume all other roles are allowed.
        return
    for role in roles:
        if role not in special_roles and role not in user_roles:
            raise 'Unauthorized', 'Too many roles specified.'


def mergedLocalRoles(object):
    """Returns a merging of object and its ancestors'
    __ac_local_roles__."""
    # Modified from AccessControl.User.getRolesInContext().
    merged = {}
    object = getattr(object, 'aq_inner', object)
    while 1:
        if hasattr(object, '__ac_local_roles__'):
            dict = object.__ac_local_roles__ or {}
            if callable(dict): dict = dict()
            for k, v in dict.items():
                if merged.has_key(k):
                    merged[k] = merged[k] + v
                else:
                    merged[k] = v
        if hasattr(object, 'aq_parent'):
            object=object.aq_parent
            object=getattr(object, 'aq_inner', object)
            continue
        if hasattr(object, 'im_self'):
            object=object.im_self
            object=getattr(object, 'aq_inner', object)
            continue
        break
    return merged


import Acquisition
from Globals import HTMLFile

class ToolInit (Acquisition.Implicit):
    '''Utility class that can generate the factories for several tools
    at once.'''

    manage_addToolForm = HTMLFile('addTool', globals())

    def __init__(self, meta_type, tools, product_name, icon):
        ''
        self.meta_type = meta_type
        self.tools = tools
        self.product_name = product_name
        self.icon = icon

    def manage_getMetaType(self):
        ''
        return self.meta_type

    def manage_getProductName(self):
        ''
        return self.product_name

    def manage_getIcon(self):
        ''
        return self.icon

    def manage_addTool(trueself, self, type, REQUEST=None):
        '''Adds the tool specified by name.'''
        obj = None
        for tool in trueself.tools:
            if tool.meta_type == type:
                obj = tool()
                break
        if obj is None:
            raise 'NotFound', type
        self._setObject(obj.id, obj)
        if REQUEST is not None:
            return self.manage_main(self, REQUEST)

    def manage_listTypes(self):
        ''
        r = []
        for tool in self.tools:
            r.append(tool.meta_type)
        return r

    def initialize(self, context):
        # Add only one meta type to the folder add list.
        context.registerClass(
            meta_type = self.meta_type,
            constructors = (self.manage_addToolForm,
                            self.manage_getMetaType,
                            self.manage_getProductName,
                            self.manage_getIcon,
                            self.manage_listTypes,
                            self.manage_addTool,),
            icon = self.icon
            )

        for tool in self.tools:
            tool.icon = 'misc_/%s/%s' % (self.product_name, self.icon)
            # context.registerBaseClass(tool)


import OFS

def initializeBasesPhase1(base_classes, module):
    """
    Executes the first part of initialization of ZClass base classes.
    Stuffs a _ZClass_for_x class in the module for each base.
    """
    rval = []
    for base_class in base_classes:
        d={}
        zclass_name = '_ZClass_for_%s' % base_class.__name__
        exec 'class %s: pass' % zclass_name in d
        Z = d[ zclass_name ]
        Z.propertysheets = OFS.PropertySheets.PropertySheets()
        Z._zclass_ = base_class
        Z.manage_options = ()
        Z.__module__ = module.__name__
        setattr( module, zclass_name, Z )
        rval.append(Z)
    return rval

def initializeBasesPhase2(zclasses, context):
    """
    Finishes ZClass base initialization.  zclasses is the list returned
    by initializeBasesPhase1().  context is a ProductContext object.
    """
    for zclass in zclasses:
        context.registerZClass(zclass)


if 0:
  # Prototype for a "UniqueId" base ZClass.
  import OFS

  class UniqueSheet(OFS.PropertySheets.PropertySheet,
                  OFS.PropertySheets.View):
    'Manage id of unique objects'

    manage = Globals.HTMLFile('uniqueid', globals())

    def getId(self):
        return self.getClassAttr('id')

    def manage_edit(self, id, REQUEST=None):
        self.setClassAttr('id', id)
        if REQUEST is not None:
            return self.manage(self, REQUEST)


  class ZUniqueObjectPropertySheets(OFS.PropertySheets.PropertySheets):

    unique = UniqueSheet('unique')

  class ZUniqueObject:
    '''Mix-in for unique zclass instances.'''

    _zclass_ = UniqueObject

    propertysheets = ZUniqueObjectPropertySheets()

    manage_options = (
        {'label': 'Id', 'action':'propertysheets/unique/manage'},
        )
