##############################################################################
# 
# Zope Public License (ZPL) Version 1.0
# -------------------------------------
# 
# Copyright (c) Digital Creations.  All rights reserved.
# 
# This license has been certified as Open Source(tm).
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
# 
# 1. Redistributions in source code must retain the above copyright
#    notice, this list of conditions, and the following disclaimer.
# 
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions, and the following disclaimer in
#    the documentation and/or other materials provided with the
#    distribution.
# 
# 3. Digital Creations requests that attribution be given to Zope
#    in any manner possible. Zope includes a "Powered by Zope"
#    button that is installed by default. While it is not a license
#    violation to remove this button, it is requested that the
#    attribution remain. A significant investment has been put
#    into Zope, and this effort will continue if the Zope community
#    continues to grow. This is one way to assure that growth.
# 
# 4. All advertising materials and documentation mentioning
#    features derived from or use of this software must display
#    the following acknowledgement:
# 
#      "This product includes software developed by Digital Creations
#      for use in the Z Object Publishing Environment
#      (http://www.zope.org/)."
# 
#    In the event that the product being advertised includes an
#    intact Zope distribution (with copyright and license included)
#    then this clause is waived.
# 
# 5. Names associated with Zope or Digital Creations must not be used to
#    endorse or promote products derived from this software without
#    prior written permission from Digital Creations.
# 
# 6. Modified redistributions of any form whatsoever must retain
#    the following acknowledgment:
# 
#      "This product includes software developed by Digital Creations
#      for use in the Z Object Publishing Environment
#      (http://www.zope.org/)."
# 
#    Intact (re-)distributions of any official Zope release do not
#    require an external acknowledgement.
# 
# 7. Modifications are encouraged but must be packaged separately as
#    patches to official Zope releases.  Distributions that do not
#    clearly separate the patches from the original work must be clearly
#    labeled as unofficial distributions.  Modifications which do not
#    carry the name Zope may be packaged in any form, as long as they
#    conform to all of the clauses above.
# 
# 
# Disclaimer
# 
#   THIS SOFTWARE IS PROVIDED BY DIGITAL CREATIONS ``AS IS'' AND ANY
#   EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
#   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
#   PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL DIGITAL CREATIONS OR ITS
#   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
#   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
#   USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
#   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
#   OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
#   OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
#   SUCH DAMAGE.
# 
# 
# This software consists of contributions made by Digital Creations and
# many individuals on behalf of Digital Creations.  Specific
# attributions are listed in the accompanying credits file.
# 
##############################################################################

import Globals
from Globals import HTMLFile, HTML
from Discussions import DiscussablePortalContent
from Products.PTKBase.PortalContent import PortalContent
from DemoDublinCore import DemoDublinCore

def addDocument(self, id, title, description,
                text_format, text, RESPONSE=None):
    """
    Add a Document
    """
    o=Document()
    o.id=id
    o.title=title
    o.description=description
    o.text=text
    o.text_format=text_format
    o._parse()
    self._setObject(id,o)
    if RESPONSE is not None:
        RESPONSE.redirect(self.absolute_url() + '/folder_contents')
    
class Document( DiscussablePortalContent
              , DemoDublinCore
              ):
    """
    A Document
    """

    meta_type='Document'
    effective_date = expiration_date = None

    __ac_permissions__=(
        ('View', ('', 'view',)),
        ('View management screens',('editForm',)),
        ('Change Documents', ('edit',), ('Owner','Manager')),
        )

    view=HTMLFile('dtml/documentView', globals())
    index_html = view
    editForm=HTMLFile('dtml/documentEdit',globals())

    def __call__(self, REQUEST, **kw):
        return apply(self.view, (self, REQUEST), kw)

    def edit(self, title, description, text_format, text, file='', REQUEST=None):
        """
        Edit the Document
        """
        self.title=title
        self.description=description
        self.text=text
        if file:
            if type(file) is not type(''):
                file=file.read()
                if file:
                    self.text=file
        self.text_format=text_format
        self._parse()
        self.reindexObject()
        if REQUEST is not None:
            return self.editForm(self, REQUEST,
                                 portal_status_message='Document changed.')

    def _parse(self):
        if self.text_format=='structured-text':
            ct = self._format_text(text=self.text)
            if type(ct) is not type(''):
                ct = ct.read()
            self.cooked_text=ct
        else:
            self.cooked_text=self.text
            
    _format_text=HTML('''<dtml-var text fmt="structured-text">''')

    def SearchableText(self):
        "text for indexing"
        return "%s %s %s" % (self.title, self.description, self.text)

    def Description(self):
        "description for indexing"
        return self.description

    def listActions(self, info):
        content_url = info.content_url
        return ( PortalContent.listActions(self, info)
               + ( { 'name'         : 'Edit'
                   , 'url'          :  content_url + '/editForm'
                   , 'permissions'  : ( 'Change Documents', )
                   , 'category'     : 'object'
                   }
                 , { 'name'         : 'Metadata'
                   , 'url'          :  content_url + '/metadata'
                   , 'permissions'  : ( 'Change Documents', )
                   , 'category'     : 'object'
                   }
                 )
               )

    ## FTP handlers

    def PUT(self, REQUEST, RESPONSE):
        """Handle HTTP (and presumably FTP?) PUT requests"""
        try:
            self.dav__init(REQUEST, RESPONSE)
            body=REQUEST.get('BODY', '')
            self.edit(self.title, self.description, self.text_format, body)
            RESPONSE.setStatus(204)
            return RESPONSE
        except:
            import traceback
            traceback.print_exc()
            raise

    def manage_FTPget(self):
        "Get the document body for FTP download"
        return self.text

Globals.default__class_init__(Document)

from Products.PTKBase.register import registerPortalContent
registerPortalContent(Document,
                      constructors=(addDocument,),
                      action='Wizards/Document',
                      icon="document.gif",
                      productGlobals=globals())
