##############################################################################
# 
# Zope Public License (ZPL) Version 1.0
# -------------------------------------
# 
# Copyright (c) Digital Creations.  All rights reserved.
# 
# This license has been certified as Open Source(tm).
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
# 
# 1. Redistributions in source code must retain the above copyright
#    notice, this list of conditions, and the following disclaimer.
# 
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions, and the following disclaimer in
#    the documentation and/or other materials provided with the
#    distribution.
# 
# 3. Digital Creations requests that attribution be given to Zope
#    in any manner possible. Zope includes a "Powered by Zope"
#    button that is installed by default. While it is not a license
#    violation to remove this button, it is requested that the
#    attribution remain. A significant investment has been put
#    into Zope, and this effort will continue if the Zope community
#    continues to grow. This is one way to assure that growth.
# 
# 4. All advertising materials and documentation mentioning
#    features derived from or use of this software must display
#    the following acknowledgement:
# 
#      "This product includes software developed by Digital Creations
#      for use in the Z Object Publishing Environment
#      (http://www.zope.org/)."
# 
#    In the event that the product being advertised includes an
#    intact Zope distribution (with copyright and license included)
#    then this clause is waived.
# 
# 5. Names associated with Zope or Digital Creations must not be used to
#    endorse or promote products derived from this software without
#    prior written permission from Digital Creations.
# 
# 6. Modified redistributions of any form whatsoever must retain
#    the following acknowledgment:
# 
#      "This product includes software developed by Digital Creations
#      for use in the Z Object Publishing Environment
#      (http://www.zope.org/)."
# 
#    Intact (re-)distributions of any official Zope release do not
#    require an external acknowledgement.
# 
# 7. Modifications are encouraged but must be packaged separately as
#    patches to official Zope releases.  Distributions that do not
#    clearly separate the patches from the original work must be clearly
#    labeled as unofficial distributions.  Modifications which do not
#    carry the name Zope may be packaged in any form, as long as they
#    conform to all of the clauses above.
# 
# 
# Disclaimer
# 
#   THIS SOFTWARE IS PROVIDED BY DIGITAL CREATIONS ``AS IS'' AND ANY
#   EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
#   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
#   PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL DIGITAL CREATIONS OR ITS
#   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
#   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
#   USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
#   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
#   OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
#   OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
#   SUCH DAMAGE.
# 
# 
# This software consists of contributions made by Digital Creations and
# many individuals on behalf of Digital Creations.  Specific
# attributions are listed in the accompanying credits file.
# 
##############################################################################
 
import Globals
from Globals import HTMLFile, HTML
from Discussions import DiscussablePortalContent
from Products.PTKBase.PortalContent import PortalContent
from DemoDublinCore import DemoDublinCore


def addLink(self, id, title, remote_url, text, RESPONSE=None):
    """
    Add a Link
    """
    o=Link()
    o.id=id
    o.title=title
    o.text=text
    o.remote_url=remote_url
    o._parse()
    self._setObject(id,o)
    if RESPONSE is not None:
        RESPONSE.redirect(self.absolute_url()+'/folder_contents')
        
class Link( DiscussablePortalContent
          , DemoDublinCore
          ):
    """
    A Link
    """

    meta_type='Link'
    effective_date = expiration_date = None

    __ac_permissions__=(
        ('View', ('', 'view')),
        ('View management screens',('editForm',)),
        ('Change Links', ('edit',), ('Owner','Manager')),
        )

    view=HTMLFile('dtml/linkView', globals())
    index_html = view
    editForm=HTMLFile('dtml/linkEdit',globals())

    def __call__(self, REQUEST, **kw):
        return apply(self.view, (self, REQUEST), kw)

    def edit(self, title, remote_url, text, REQUEST=None):
        """
        Edit the Link
        """
        self.title=title
        self.text=text
        self.remote_url=remote_url
        self._parse()
        self.reindexObject()
        if REQUEST is not None:
            return self.editForm(self, REQUEST,
                                 portal_status_message='Link changed.')

    _format_text=HTML("""<dtml-var text fmt="structured-text">""")

    def _parse(self):
        self.cooked_text=self._format_text(self) 

    def SearchableText(self):
        "text for indexing"
        return "%s %s" % (self.title, self.text)

    def Description(self):
        "description for indexing"
        return self.text

    def listActions(self, info):
        content_url = info.content_url
        return ( PortalContent.listActions(self, info)
               + ( { 'name'         : 'Edit'
                   , 'url'          :  content_url + '/editForm'
                   , 'permissions'  : ( 'Change Links', )
                   , 'category'     : 'object'
                   }
                 , { 'name'         : 'Metadata'
                   , 'url'          :  content_url + '/metadata'
                   , 'permissions'  : ( 'Change Links', )
                   , 'category'     : 'object'
                   }
                 )
               )

Globals.default__class_init__(Link)

from Products.PTKBase.register import registerPortalContent
registerPortalContent(Link,
                      constructors=(addLink,),
                      action='Wizards/Link',
                      icon="link.gif",
                      productGlobals=globals())
